/*___________________________________________________________________________*/
/* IBM Optimization Solutions -- Stochastic Solutions/Extensions             */
/* ekkstoch.h -        C header file                                         */
/* (c) IBM Corp, 1990,1997,1998, 1999 All rights reserved.                   */
/*                                                                           */
/* US Government Users Restricted Rights - Use, duplication, or disclosure   */
/* restricted by GAS ADP Schedule Contract with IBM Corp.                    */
/*___________________________________________________________________________*/

#ifdef __cplusplus
extern "C"{
#endif


#ifndef OSLLIBAPI
#ifdef OSLMSDLL
#define OSLLINKAGE __stdcall
#define OSLLINKAGE_CB __stdcall
#define OSLLIBAPI __declspec(dllimport)
#endif
#ifdef OSLMSSTAT
#define OSLLINKAGE_CB __stdcall
#define OSLLINKAGE __stdcall
#define OSLLIBAPI
#endif
#ifdef OSLWCSTAT
#define OSLLINKAGE_CB
#define OSLLINKAGE
#define OSLLIBAPI
#endif
#endif

/* Date: March 1998
 *
 *      User access to Stochastic structures:
 *         usrCoreData structure is generated by ekks_GetCoreData.
 *         usrNodeData structure is generated by ekks_GetNodeData.
 *         usrStoch structure (for StochData or StochModel) is generated by
ekks_GetScenarioTree.
 *
 *      NB: usrNodeData and usrStoch structures are in volatile memory,
 *          but usrCoreData is permanent.
 */

#ifndef USRSTOC_H
#define USRSTOC_H
#include <stdarg.h>

#ifndef DISTRIB_H
#define DISTRIB_H

#define DISTN_NOT_IMPLEMENTED   0
#define SCENARIOS               1
#define INDEP_DISCRETE          2
#define BLOCK_DISCRETE          3

typedef void Distribution;

typedef struct {
        long type,stag;
        Distribution *dist;
} IndepRVInfo;

typedef struct {
        long nrvar;
        IndepRVInfo **rvar;
} Independent;

typedef struct{
        long *nind;
        IndepRVInfo *rvar;
        long nels;
        long *mrow,*mcol;
        double *dels;
} LinTr;

typedef struct {
        long nevt,*evst,*mrow,*mcol;
        double *dprb, *dels;
} Discrete;

typedef struct{
        long irow,icol;
        double param1,param2;
} TwoParam ;

#endif



typedef struct {
        long nrow;                      /* number of rows */
        long ncol;                      /* number of columns */
        long nels;                      /* number of matrix elements */
        long nstg;                      /* number of stages */
        long nblocks;                   /* number of blocks in matrix */
        double *drlo,*drup;             /* row bounds */
        double *dclo,*dcup;             /* column bounds */
        double *dobj;                   /* objective */
        long *mrow;                     /* matrix row indices */
        long *mcol;                     /* matrix column indices */
        double *dels;                   /* matrix elements */
        long *rowstart;                 /* starts for stage partition of rows */
        long *colstart;                 /* starts for stage partition of cols */
        long *elemstart;                /* starts for stage partition of matrix*/
                                        /*   Matrix is partitioned by row blocks
                                             and then by column blocks thus
 *                                           t*(t+1)/2-1 is the matrixindex for
                                             the start of the t-th stage node block
                                         *   (as would be obtained by numbering blocks
                                         *   starting from the top and moving left to
                                         *   right).  The matrix is assumed full lower
                                         *   block triangular.*/
        long *sortin2exr;               /* in-to-external row index translation*/
        long *sortin2exc;               /* in-to-external col index translation
*/
} usrCoreData;

typedef struct {
        long stg;                       /* stage of node ( = number of partitions) */
        long nrow;                      /* number of rows */
        long ncol;                      /* number of columns */
        long nels;                      /* number of node block elements */
        double *drlo,*drup;             /* row bounds */
        double *dclo,*dcup;             /* column bounds */
        double *dobj;                   /* objective */
        long *mrow;                     /* block row indices */
        long *mcol;                     /* block column indices */
        double *dels;                   /* block elements */
        long *elemstart;                /* starts for stage partition of node block:
                                           thus t-1 is the index for the start of the
                                           t-th stage arc block and stg-1 is the
                                           index for the start of the node block.
                                           The node block has (poss.  empty) arc
                                           blocks for all stages t < istg. */
} usrNodeData;

typedef struct {
        double prob;            /* probability of node in model */
        long orig;              /* original node name (numbered from 1) */
        long par;               /* parent's position in list (beg=1) */
        long child;             /* next-stage child (end=0) */
        long sib_cous;          /* same-stage sibling or cousin (end=0) */
        long stag;              /* original time stage */
        long scen;              /* original scenario number */
        long rbas;              /* row offset in matrix */
        long cbas;              /* column offset in matrix */
        long nrow;              /* number of rows */
        long ncol;              /* number of columns */
        long isvirtual;         /* = 1 if node not actually in model */
        void *user;             /* (optional) pointer to user structure */
} usrNode;

typedef struct {
        long nnode;             /* number of nodes */
        usrNode *node;          /* array of node structures */
        long nleaf;             /* number of leaves */
        long *leaf;             /* list of leaf positions in node array (beg=1)
*/
        void *user;             /* (optional) pointer to user structure */
} usrStoch;

/* Replace/Add core values */

#define REPLACE_CORE_VALUES 1
#define ADD_TO_CORE_VALUES  0

/* stubs for access to Stochastic internal structures */
#ifndef STOC_H
        typedef char Stoch;
        typedef Stoch Stoch_Data;
        typedef Stoch Stoch_Model;

        typedef enum {linear=1, quadratic=2} CoreType;
#endif

#ifndef USRTREE_H
#ifndef TREE_H
typedef char Tree;      /* generic tree object is a pointer to Tree
**********************************************************************/
#endif
#endif

/*********************************************************************/


extern OSLLIBAPI int OSLLINKAGE ekks_InitializeOSL();
/*

 *  This routine will be removed in the next release level of Stochastic Solutions.
 *  It remains here until then for compatibility. Use ekks_Initilize (); instead.
 *  RETURNS:
 *       value of return code
 **********************************************************************/

extern OSLLIBAPI double * OSLLINKAGE ekksOSL_GetDspace(long maxspc, long maxmod);
/*

 *  ekksOSL_GetDspace will cause Stochastic Extensions to malloc DSPACE of
 *  size sizeof(double)*maxspc. The number of models to be passed to OSL is
 *  specified in maxmod. The address of DSPACE is returned.
 *
 *  INPUT:
 *   long maxspc            number of doublewords of DSPACE to malloc
 *   long maxmod            number of stochastic models to specify to OSL
 *

 *  RETURNS:
 *       address of DSPACE
 *
 * An estimation formula:
 * ======================
 *
 * If
 *   ncols = #columns in core,
 *   nrows = #rows in core,
 *   nelems = #elements in core,
 *   maxmod = # OSL models (Benders' subproblems + 1),
 *   maxiters = # of Benders' iterations,
 *
 * Then, maxspc may be estimated (for ekks_BendersLSolve() ) as:
 *
 *   maxspc = 100000 +
 *            15*(nelems+ncols+nrows) + 15*(ncols+1) * maxmod * maxiters
 *
 **********************************************************************/

extern OSLLIBAPI double * OSLLINKAGE ekksOSL_CreateDspace(long maxspc, long maxmod);
/*

 *  ekksOSL_CreateDspace will cause Stochastic Extensions to malloc DSPACE of
 *  size sizeof(double)*maxspc. The number of models to be passed to OSL is
 *  specified in maxmod. The address of DSPACE is returned.
 *
 *  INPUT:
 *   long maxspc            number of doublewords of DSPACE to malloc
 *   long maxmod            number of stochastic models to specify to OSL
 *

 *  RETURNS:
 *       address of DSPACE
 *
 * An estimation formula:
 * ======================
 *
 * If
 *   ncols = #columns in core,
 *   nrows = #rows in core,
 *   nelems = #elements in core,
 *   maxmod = # OSL models (Benders' subproblems + 1),
 *   maxiters = # of Benders' iterations,
 *
 * Then, maxspc may be estimated (for ekks_BendersLSolve() ) as:
 *
 *   maxspc = 100000 +
 *            15*(nelems+ncols+nrows) + 15*(ncols+1) * maxmod * maxiters
 *
 **********************************************************************/

extern OSLLIBAPI double * OSLLINKAGE ekks_GetDspace(Stoch_Model *stoch);
/*

 *  ekks_GetDspace will cause Stochastic Extensions to return the address of
 *  DSPACE associated with the Stoch_Model object stoch.
 *
 *  INPUT:
 *   Stoch_Model *stoch     address of the Stoch_Model object stoch
 *

 *  RETURNS:
 *       address of DSPACE
 *
 **********************************************************************/

extern OSLLIBAPI int OSLLINKAGE ekks_Initialize();
/*

 *  Initializes Stochastic data structures.
 *
 * RETURNS:
 *      integer return code
 **********************************************************************/

extern OSLLIBAPI long OSLLINKAGE ekks_GetNumSubproblems(Stoch_Model *stoch);
/*

 *  ekks_GetNumSubproblems will cause Stochastic Extensions to return the
 *  number of stochastic models associated with the data of the Stoch_Model
 *  object stoch.
 *
 *  INPUT:
 *   Stoch_Model *stoch     address of the Stoch_Model object stoch
 *

 *  RETURNS:
 *       number of stochastic models associated with the data of the
 *       Stoch_Model object stoch.
 *
 **********************************************************************/
extern OSLLIBAPI void OSLLINKAGE ekks_AddEvent
                (long *rc, Stoch_Data *stoch, Tree **treep, long *index,
                 double dp, long replace );
/*

 *  Takes an event for a INDEP_DISCRETE distribution and adds it
 *  as a scenario.
 **
 *
 * INPUT:
 *  Stoch_Data *stoch      pointer to a Stoch_Data structure.
 *  tree **treep           address of pointer to event tree under
 *                         construction (pointer must be NULL for first call)
 *  long *index            pointer to event indices.
 *  double dp              probability
 *  long replace           0=add to, 1=replace core values
 *
 * RETURNS:
 *      long *rc           pointer to return code (2 is fatal)
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_AddScenario
                (long *rc, Stoch_Data *stoch, long nscn, long ianc, long istg,
                 double dprob, long itype, long nrow,long ncol,long nels,
                 double *dobj, double *drlo, double *drup, double *dclo, double *dcup,
                 long *mrow, long *mcol, double *dels, long replace );
/*

 *      Adds a scenario to stochastic problem, sets the scenario
 *      probability, the branching information, and stores the scenario data
 *      by node in an SPL-file.
 **
 *      Errors occur when branching information is inconsistent
 *      or when dimensions are inconsistent with the core.
 *
 *      If this call is repeated for the same scenario, the scenario
 *      probability is incremented by the amount of the new probability.
 *      No other data is changed.
 *
 *      All scenario LP data is ADDED to the corresponding core data when
 *      forming an LP.  This requires a convention for dealing with infinite
 *      bounds in the core.  If the core value is infinite and the incoming
 *      scenario value is nonzero, then a warning message is issued and the
 *      incoming scenario value is set to zero.
 *
 *      CORE VALUE      SCENARIO VALUE     ACTION              MESSAGE    RC
 *
 *      finite          infinite           none                none        0
 *      infinite        nonzero            scen value zeroed   warning     1
 *
 *      The LP data is NOT kept in core memory.  (It may be retrieved for
 *      the user by a call to ekks_GetNodeData.)  The LP matrix must be in
 *      "indices" sparse format.
 *
 *   If replace = 0:
 *
 *      LP data should be the "difference" of the intended scenario LP data
 *      with the core LP data.
 *
 *   If replace = 1:
 *
 *      Scenario LP data should "replace" the core LP data.  In this case,
 *      the subroutine calculates the difference for you.
 *
 *
 * INPUT:
 *      Stoch_Data *stoch         pointer to Stoch_Data structure initialized by
 *                                ekks_InitializeApplication
 *      long nscn                 scenario number (1,2,...)
 *      long ianc                 ancestor scenario number
 *      long istg                 branching stage number
 *      double dprob              probability of scenario
 *      long itype                matrix type (MUST be 1 - by indices format)
 *      long nrow                 number of rows (must match core model)
 *      long ncol                 number of columns (must match core model)
 *      long nels                 number of elements in scenario matrix
 *      double *dobj              array of objective entries
 *      double *drlo              array of row lower bound entries
 *      double *drup              array of row upper bound entries
 *      double *dclo              array of column lower bound entries
 *      double *dcup              array of column upper bound entries
 *      long *mrow                array of matrix row indices
 *      long *mcol                array of matrix column indices
 *      double *dels              array of matrix entries
 *      long replace              0=add to, 1=replace core values
 *
 * Output:
 *      long *rc                  pointer to return code (2 is fatal)
 *
 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_BendersLSolve
                (long *rc, Stoch_Model *omodel, long ipcut, long maxmod, long maxits,
                 double rmaxmin, long inwmt, long icrsh, long ilpdc, long iscal,
                 long iprsl, long iprsltype, long iprslunit, long isalg, long largebounds,
                 long *status, double *optgap );
/*

 *  Applies Benders/L-shaped decomposition to the stochastic model,
 *  with cut made at the stage number IPCUT.
 **
 *
 * METHODOLOGY:
 *
 *  A single master problem is created and up to MAXMOD subproblems
 *  corresponding to a cut made at stage IPCUT; that is, the master
 *  problem contains all rows and columns for stages prior to IPCUT.
 *  If MAXMOD is not sufficient to hold all the subproblems, the
 *  subproblems are distributed as evenly as possible among MAXMOD models.
 *
 *  The program performs MAXITS iterations of Benders decomposition,
 *  terminating if optimality, infeasibility, or unboundedness is
 *  detected.
 *
 * INPUT:
 *
 *  Stoch_Model *oModel   pointer to stochastic model created by OSL.
 *  long ipcut            period at which Benders cuts are made
 *  long maxmod           maximum number of subproblems
 *  long maxits           maximum number of Master iterations
 *  double rmaxmin        maximize = -1.0, minimize = 1.0
 *  long inwmt            compress model matrices into a new copy, 0 = don't compress
 *  long iscal            0=don't, 1=do perform scaling for each submodel
 *  long icrsh            type of crash to perform before first solution of submodels (0 = none)
 *  long ilpdc            0=don't, 1=do LP decomposition for each submodel
 *  long iprsl            0=don't, 1=do presolve each submodel
 *  long iprsltype        type of presolve
 *  long iprslunit        presolve unit
 *  long isalg            EKKSSLV algorithm for first solution of submodels
 *  long large_bounds     0=don't, 1=do add large bounds to master
 *  double *optgap        pointer to maximum optimality gap before stopping BendersLSolve
 *
 * OUTPUT:
 *
 *  long *rc             pointer to return code, >= 2 indicates fatal error.
 *  long *status         0=optimal,1=infeasible,2=unbounded,3=unfinished
 *  double *optgap       pointer to Optimality gap at termination
 *
 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_GetCoreData
                (long *rc, Stoch_Data *stoch, usrCoreData **pcore );
/*
 *   Copy core data pointers into the usrCoreData structure.
 **
 *   If *pcore is NULL, then a new usrCoreData structure is malloc'ed
 *   and the pointer returned.
 *
 *   The data pointed to by usrCoreData pointers is stable.
 *   (It is the original data as formed by ekks_CreateCore.)
 *
 *  INPUT:
 *   Stoch_Data *stoch      pointer initialized by ekks_InitializeApplication
 *                          and ekks_CreateCore
 *
 *  OUTPUT:
 *   usrCoreData **pcore    address of pointer to type usrCoreData.
 *
 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_CreateCore
                (long *rc, Stoch_Data *stoch, long nstg, long *Rstag, long *Cstag,
                 long irow, long icol, long iels, double *dobj, double *drlo,
                 double *drup, double *dclo, double *dcup, long *mrow, long *mcol,
                 double *dels, double *dqdg);
/*

 * Sorts core model row and column indices by stages, groups core model data
 * into blocks corresponding to stages, and stores all model information, sort
 * keys, and grouped data into an SPL file.
 *
 * The data may be accessed by a ekks_GetCoreData call.
 *
 * INPUT:
 *
 *      Stoch_Data stoch         pointer to Stoch_Data structure initialized by
                                 ekks_InitializeApplication
 *      long nstg                number of stages in the core model
 *      long *Rstag              array of stage numbers for rows
 *      long *Cstag              array of stage numbers for columns
 *      long irow                number of rows in core model
 *      long icol                number of columns in core model
 *      long iels                number of matrix elements in core model
 *      double *dobj             array of objective entries
 *      double *drlo             array of row lower bound entries
 *      double *drup             array of row upper bound entries
 *      double *dclo             array of column lower bound entries
 *      double *dcup             array of column upper bound entries
 *      long *mrow               array of matrix row indices
 *      long *mcol               array of matrix column indices
 *      double *dels             array of matrix entries
 *      double *dqdg             array of quadratic matrix diagonal entries
 *
 * OUTPUT:
 *
 *      long *rc                 pointer to return code, 2 is fatal
 *
 * Returns:
 *      void
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_ChangeProbability
                ( long *rc, Stoch_Data *stoch, double *usrProb );

/*
 *    Updates leaf node probabilities with new scenario probabilities.
 *
 *    Note: The original scenario probabilities are unaffected.
 *          These can be restored by a call to ekks_ChangeProbability with a
 *          NULL usrProb parameter.
 *
 * INPUT:
 *      Stoch_Data *stoch         pointer to Stoch_Data structure initialized by
 *                                ekks_InitializeApplication.
 *      double *usrProb           pointer to array of new probabilities.
 *
 * OUTPUT:
 *      long *rc                  pointer return code, 2 is fatal
 *
 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_GetPointerToDistribution
                ( long *rc, Stoch_Data *stoch, Independent **s_indep );
/*

 *  Mallocs an independent data structure and sets it to point to the
 *  information collected in a ekks_ReadMPS reading of an INDEPENDENT/DISCRETE
 *  format.
 *
 * INPUT:
 *  Stoch_Data *sdata          pointer to a Stoch_Data structure.

 *
 * OUTPUT:
 *  Independent **s_indep      pointer to an Independent structure.
 *
 *  RETURNS:
 *       long *rc              pointer to return code, 2 is fatal

 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_DescribeModel
                (long *rc, Stoch_Data *stoch, long nnodes, long *usrndlist,
                 Stoch_Model **smodel, long *nblocks, long calcprob );
/*

 *      Creates model node information structures and offsets from
 *      user-supplied nodelist.
 **
 *      If nodelist is empty (nnodes=0) then all nodes in the Stoch_Data
 *      structure are included.
 *
 *      Node probabilities are inherited from Stoch_Data structure.
 *      Zero probability nodes (and their incoming arcs) are removed.
 *
 *      Virtual nodes (ancestors not in model one of whose children are)
 *      are recorded and are offset by the total number of real node columns.
 *
 *      If calcprob is set, the probabilities for each node are
 *      recomputed and normalized so that the probability totals to one.
 *
 *      No matrix data is passed.
 *      The number of blocks to be passed is returned.
 *
 * INPUT:
 *      Stoch_Data *stoch         pointer to Stoch_Data structure
 *      long nnodes               number of nodes
 *      long *usrndlist           pointer to list of nodes to be included in model
 *      long calcprob             0=don't, 1=do  recompute node probabilities.
 *
 * OUTPUT:
 *      Stoch_Model **smodel      pointer to new stochastic model
 *      long *nblocks             number of blocks in model
 *
 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_GetPointerToSolution
                (long *rc, Stoch *stoch, long *nmod, double **dsoln);
/*
 *  Gets a pointer to the solution region for the stochastic model nmod.
 **
 *
 * INPUT:
 *  Stoch *stoch           pointer to the Stoch_Model or Stoch_Data structure
 *  long *nmod             pointer to the number of the model
 *
 * OUTPUT:
 *      double **dsoln     pointer to the solutions region
 *      long *rc           pointer to return code (2 is fatal)

 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_GetPointerToRowSolution
                (long *rc, Stoch *stoch, long *nmod, double **rsoln, long *nrow);
/*
 *  Gets a pointer to the solution region for the stochastic model nmod.
 **
 *
 * INPUT:
 *  Stoch *stoch           pointer to the Stoch_Model or Stoch_Data structure
 *  long *nmod             pointer to the number of the model
 *
 * OUTPUT:
 *      double **rsoln     pointer to the solutions region
 *      long *rc           pointer to return code (2 is fatal)
 *      long *nrow         pointer to the number of rows

 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_GetPointerToReducedCosts
                (long *rc, Stoch *stoch, long *nmod, double **redcosts, long *ncol);
/*
 *  Gets a pointer to the solution region for the stochastic model nmod.
 **
 *
 * INPUT:
 *  Stoch *stoch           pointer to the Stoch_Model or Stoch_Data structure
 *  long *nmod             pointer to the number of the model
 *
 * OUTPUT:
 *      double **redcosts  pointer to the solutions region
 *      long *rc           pointer to return code (2 is fatal)
 *      long *ncol         pointer to the number of columns

 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_GetPointerToRowDuals
                (long *rc, Stoch *stoch, long *nmod, double **rowduals, long *nrow);
/*
 *  Gets a pointer to the solution region for the stochastic model nmod.
 **
 *
 * INPUT:
 *  Stoch *stoch           pointer to the Stoch_Model or Stoch_Data structure
 *  long *nmod             pointer to the number of the model
 *
 * OUTPUT:
 *      double **rowduals  pointer to the solutions region
 *      long *rc           pointer to return code (2 is fatal)
 *      long *nrow         pointer to the number of rows

 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekks_GetSolution
                (long *rc, Stoch *stoch, long s, long colrowtype,
                 double *solution, long *indx);
/*
 *  Gets a solution for a scenario.

 *
  * INPUT:
 *  Stoch_Model *smodel          Stoch_Model or Stoch_Data pointer to be deleted
 *..long s                       Scenario number to return
 *  long colrowtype              Type of solution (1=rows, else columns) to be retrieved
 *
 * OUTPUT:
 *  double *solution             pointer to the solution
 *  long *indx                   pointer to the indices
 *  long *rc                     pointer to return code, 2 is fatal

 *
 * RETURNS:
 *      void
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_FreeStructure
                (Stoch *stoch);
/*
 *  Free all data structures and temporary files associated with the Stoch_Data or
 *  Stocg_Model structure.
 *
 * INPUT:
 *  Stoch *stoch           pointer to Stoch_Model or Stoch_Data to be deleted
 *
 **********************************************************************/


extern OSLLIBAPI long OSLLINKAGE ekks_FreeUsrCore(usrCoreData *uCore);
/*

 *  Delete the uCore object and free up the allocated memory area.
 *
 *  RETURNS:
 *       value of return code
 **********************************************************************/

extern OSLLIBAPI long OSLLINKAGE ekks_FreeUsrTree(Tree *uTree);
/*

 *  Delete the uTree object and free up the allocated memory area.
 *
 *  RETURNS:
 *       value of return code
 **********************************************************************/


extern OSLLIBAPI void OSLLINKAGE ekksOSL_FreeDspace();

/*
 *  Free OSL workspace and reinitialize OSL parameters.
 *
 *
 **********************************************************************/


extern OSLLIBAPI void OSLLINKAGE ekks_GetStochasticData
                (long *rc, Stoch_Data *stoch, const char *filename );
/*
 *   Copies Stoch_Data structures from SPL file.
 **
 * INPUT:
 *   Stoch_Data *stoch     pointer to Stoch_Data structure initialized by
 *                         ekks_InitializeApplication.
 *   char *filename        name of SPL file.
 *
 * OUTPUT:
 *   long *rc              return code (2 is fatal)
 *
 * RETURNS:
 *      void
 *
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_InitializeApplication
                (long *rc, Stoch_Data **stoch, long maxscn );
/*
 *      Allocates pointers for stochastic data structures,
 *      opens SPL file, returns Stoch_Data pointer.
 **
 *  INPUT:
 *      long maxscn                maximum number of scenarios
 *
 *  OUTPUT:
 *      Stoch_Data **stochdata     pointer to Stoch_Data structure
 *      long *rc                   pointer to return code, 2 is fatal
 *
 *  RETURNS:
 *       void
 *
 **********************************************************************/


extern OSLLIBAPI void OSLLINKAGE ekks_GetNodeData
                (long *rc, Stoch_Data *stoch, long node, usrNodeData **punode );
/*
 *   Get stochastic node data into usrNodeData structure.
 **
 *   Allocates new usrNodeData structure.
 *   Accesses stochastic matrix and costs/bounds data from SPL file
 *   and places them in volatile working memory.  Costs and bounds are
 *   expanded to their dense form.  usrNodeData pointers are appropriately
 *   set.
 *
 *   NB: ALL matrix and costs/bounds arrays are volatile!  The next call
 *       to a Stochastic Solutions/Extensions routine WILL OVERWRITE THEM!
 *       (To protect yourself, copy them to a memory region that you maintain.)
 *
 *   NB: The matrix and costs/bounds arrays contain only the stochastic
 *       part of the data.  They must be ADDED to the relevant core data.
 *
 * INPUT:
 *       Stoch_Data *stoch     pointer to Stoch_Data structure initialized by
 *                             ekks_InitializeApplication
 *       long node             node number to be accessed
 *
 * OUTPUT:
 *       usrNodeData **punode  pointer to usrNodeData
 *       long *rc              pointer to return code, 2 is fatal
 *
 * RETURNS:
 *      void
 **********************************************************************/


extern OSLLIBAPI void OSLLINKAGE ekks_PutStochasticData
                (long *rc, Stoch *stoch, char *filename);
/*

 *  Puts the Stoch_Data structure into an SPL file to be called "filename".
 **
 *  Opens the file as a buffered direct access file, writes the data and
 *  then closes it.
 *
 *  File may be read by a call to ekks_GetStochasticData().
 *
 * INPUT:
 *   Stoch_Data *sdata     stochastic data pointer initialized by
 *                         ekks_InitializeApplication.
 *   char *filename        name of SPL file.
 *
 * OUTPUT:
 *   long *rc              pointer to return code, 2 is fatal
 *
 * RETURNS:
 *      void
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_GetScenarioTree
                (long *rc, Stoch *stoch, usrStoch **pstree );
/*

 *  Creates a user accessible stochastic structure.
 **
 *  The user needs to copy over the information
 *  to preserve it beyond the next Stochastic Solution call.
 *
 *  The input can be either a Stoch_Data structure or a Stoch_Model
 *  structure, in which case information for only those nodes in the
 *  model is compiled.
 *
 * INPUT:
 *      Stoch *stoch         pointer to a Stoch_Data or a Stoch_Model structure.
 * OUTPUT:
 *      usrStoch **pstree    pointer to a usrStoch structure.
 *      NB: arrays pointed to by pstree are volatile!
 *
 * RETURNS:
 *      long *rc             pointer to return code, 2 is fatal
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_ReadMPS
                (long *rc, Stoch_Data *stoch, long *smpstype,
                 long *replace, char *corefile, char *timefile, char *stochfile );
/*

 * Processes the Stochastic MPS files <app>.core, <app>.time and <app>.stoch,
 * creates scenario data and stores it in the StochData structure.
 **
 * The value of replace is set to 0 if STOCH file contains 'ADD' at
 * position 40 on the second line (fixed format) or 'ADD' is the third field
 * on the second line (free format).
 *
 * INPUT:
 *
 *   Stoch_Data *stoch     pointer to Stoch_Data structure to be filled in by
 *                         ekks_ReadMPS.
 *   char *corefile        pointer to the core MPS filename.
 *   char *timefile        pointer to the time MPS filename.
 *   char *stochfile       pointer to the stoch MPS filename.
 *
 * OUTPUT:
 *   long *smpstype        integer identifying type of SMPS distribution.
 *   long *replace         0=add to, 1=replace core values
 *   long *rc              pointer to return code, 2 is fatal
 *
 * RETURNS:
 *      void
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_CreateSMPS
                (long *rc, Stoch_Data *stoch, long smpstype,
                 long replace, char *stochfile );
/*

 * Creates the Stochastic MPS files <stochfile>.core, <stochfile>.time and <stochfile>.stoch
 * from the scenario data in the StochData structure.
 **
 * The type of distribution is derived from the value of smpstype.  The add/replace keyword
 * in the STOCH file is derived from the value of replace.
 *
 * INPUT:
 *
 *   Stoch_Data *stoch     pointer to Stoch_Data structure.
 *   long smpstype         integer identifying type of SMPS distribution.
 *   long replace          setting of add/replace keyword in file: 0=add to, 1=replace core values
 *   char *stochfile       pointer to the base MPS filename.
 *
 * OUTPUT:
 *   long *rc              pointer to return code, 2 is fatal
 *
 * RETURNS:
 *      void
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_ProblemType
                ( long *rc, Stoch_Data*stoch, CoreType type );
/*

 *      Sets type of core model.  Valid arguments are: linear, quadratic.
 **
 * INPUT:
 *      CoreType type             type of core model
 *      Stoch_Data *stochdata     pointer to Stoch_Data structure
 *
 * OUTPUt:
 *      long *rc                  pointer to return code (2 is fatal)
 *
 * RETURNS:
 *      void
 **********************************************************************/



#ifndef USRTREE_H
#define USRTREE_H

/***********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_InitializeTree
                (long *rc, Tree **tree,long *arr, long narr, long mxsmp, double dp );
/*

 * Creates tree and initializes structures.
 * Creates first scenario from data values in arr[].
 **
 * INPUT:
 *   long *arr[]          integer array of data values
 *   long narr            number of data values (stages) in arr[]
 *   long mxsmp           maximum number of samples to be stored
 *   long dp              probability of events
 *
 * OUTPUT:
 *   Tree *tree           pointer to tree structure
 *   long *rc             pointer to return code 0
 *
 * RETURNS:
 *      void
 **********************************************************************/

extern OSLLIBAPI long OSLLINKAGE ekks_AddToTree
                (long *rc, Tree *tree, long*arr, long narr, double dp);
/*

 * Updates scenario tree with data values in arr[].
 **
 * If arr[] is already in tree, then it increments counters
 * and returns the original scenario number,
 * otherwise it creates new scenario from data values in arr[]
 * and returns the newly created scenario number.
 *
 * INPUT:
 *     Tree *tree       pointer to tree initialize by ekks_InitializeTree.
 *     long *arr[]      integer array of data values
 *     long narr        number of data values (stages) in arr[]
 *     double dp        probability of events
 *
 * OUTPUT:
 *      long *rc        pointer to return code:
 *                         1 if maximum number of samples exceeded
 *                         2 if tree not initialized or corrupted
 *
 * RETURNS:
 *      scenario number as described above

 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE ekks_GetScenarioFromTree
                (long *rc, Tree *tree,long nsmp, long *ianc,
                 long *istg, double *wgt, long *nscn, long **arr);
/*

 * Finds scenario information for sample number nsmp.
 **
 * NB: if branching stage, "istg", has value 1, this means
 *     that the scenario branches from the root (ianc=0).
 *
 * INPUT:
 *   tree tree      pointer to tree initialized by ekks_InitializeTree.
 *   long nsmp      sample number (first sample is 0, etc)
 *
 * OUTPUT:
 *   long *ianc     pointer to ancestor scenario number.
 *   long *istg     pointer to branching stage number
 *   double *wgt    scenario weight
 *   long *nscn     scenario number
 *   long **arr       pointer to data array for scenario
 *   long *rc       pointer to return code:
 *                     1 if maximum number of samples exceeded
 *                     2 if tree not initialized or corrupted
 *
 * RETURNS:
 *      void
 **********************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_DefineParametricModel
                (long *rc, Stoch_Model *smodel,double *cdelta, double *lrdelta,
                 double *urdelta, double *lcdelta, double *ucdelta, long mask );
/*
 * Create parametric stochastic model
 *
 *  The user supplies arrays which are to be *  used to perturb the objective
 *  and bounds of the CORE model.
 *  These then are sorted and replicated to produce perturbations
 *  for the stochastic model (objective perturbations are multiplied
 *  by the node probabilities).  The CORE perturbation is replicated
 *  for each node of the stochastic model.
 *
 *  INPUT:
 *
 *  Stoch_Model *smodel    pointer to stochastic model initialized by OSL.
 *  double *cdelta         array of objective perturbations.
 *  double *lrdelta        array of lower row bound perturbations.
 *  double *urdelta        array of upper row bound perturbations.
 *  double *lcdelta        array of lower column bound perturbations.
 *  double *ucdelta        array of upper column bound perturbations.
 *  long mask              says which perturbations are used:
 *                           mask = C*1 + LR*2 + UR*4 + LC*8 + UC*16
 *                           where C,LR,UR,LC,UC are 1 or 0 depending on
 *                           whether there is or is not a perturbation of
 *                           the corresponding objective or bound.
 *
 * OUTPUT:
 *  long *rc               pointer to return code (2 is fatal)
 *
 * RETURNS:
 *      void
 *
 ***********************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_SolveParametricModel
                (long *rc, Stoch_Model *smodel, double lambda, double lambdadelta,
                 double lambdalim,long mask, long (*report) (double *) );
/*
 * Solve parametric model. If lambda is non-zero, then lambda times
 * the perturbation vector is added to the original and the resulting
 * problem is solved with EKKSSLV.
 * Thereafter, the perturbation is added in increments multiplied by
 * lambdadelta up to the maximum lambdalim.
 *
 * It is more efficient to set up the problem so that the original is solved
 * with ekkse_NestedLSolve(), then call ekkse_SolveParamentricModel() with
 * lambda=0.
 *
 * Each time the incremented problem is solved, the function pointed to
 * by report() is called with the OSL dspace attached to smodel as
 * argument.
 *
 * The mask argument is used in the same fashion as in  * ekkse_DefineParametricModel().
 *
 * INPUT:
 *   Stoch_Model *smodel           pointer to Stoch_Model
 *   double lambda                 amount of perturbation added at first
 *   double lambdadelta            incremental amount of perturbation added
 *   double lambdalim              maximum perturbation added
 *   long mask                     see ekkse_DefineParametricModel()
 *   long (*report) (double *))    called after solution
 *
 * OUTPUT:
 *   long *rc                      pointer to return code, 2 is fatal
 *
 * RETURNS:
 *      void
 *
 *******************************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_CutModelAtStage
                (long *rc, Stoch_Model *oModel, long ipcut, long maxmodels);
/*
 *   Performs a Benders cut on smodel before stage ipcut, creating node
 *   lists for at most maxmodels problems.  The first list is for the root
 *   subproblem, and consists of all (nonvirtual) nodes having stage less
 *   than ipcut.  The rest of the nodes are distributed among the leaf
 *   subproblems in such a way that the difference in the number of branches
 *   allocated between any two subproblems is no more than one.
 *
 *
 * INPUT:
 *
 *     Stoch_Model oModel       Stoch_Model to be cut
 *     long ipcut               period at which model is cut
 *     long *maxmodels          maximum number of models (Master + Subs)
 *
 * OUTPUT:
 *     long *rc                 pointer to return code, 2 is fatal.
 *
 * RETURNS:
 *      void
 ******************************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_CutModelByRow
                (long *rc, Stoch_Model *s_model, long minNumRows, long **masters);

/*
 *   Generates "depth-first" partition of the model.  Size of models
 *   is controlled by minNumRows.
 *
 *   The method adds a scenario to a model, then starting from the leaf
 *   and going back to the root all children of the parent of the
 *   node are added until minNumRows is reached.

 *
 * INPUT:
 *
 *     Stoch_Model smodel         Stoch_Model to be cut
 *     minNumRows                 min number of rows per model
 *
 * OUTPUT:
 *     long *masters              list of master nodes for each submodel
 *     long *rc                   pointer to return code, 2 is fatal.
 *
 * RETURNS:
 *      void
 **********************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_CutModelByNode
                (long *rc,Stoch_Model *s_model, long maxNodes, long **masters);

/*
 *   Generates "depth-first" partition of the model.  Number of nodes
 *   is controlled by maxNodes.  The minimum size of a model is
 *   determined by dividing .9*maxNodes into the number of rows of the
 *   problem.
 *
 *   The method adds a scenario to a model, then starting from the leaf
 *   and going back to the root all children of the parent of the
 *   node are added until the minimum size is reached.
 *
 *   Then models with smaller than minimum are added to their masters,
 *   until maxNodes models remain.

 *
 * INPUT:
 *
 *     Stoch_Model *s_model        Stoch_Model to be cut
 *     long maxNodes               maximum number of nodes to be generated
 *
 * MODIFIED:
 *     long **masters              list of master nodes for each submodel
 *
 * OUTPUT:
 *     long *rc                    pointer to return code, 2 is fatal.
 *
 * RETURNS:
 *      void
 ***********************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_SetMessage
                (long *rc, long msgid, long onoff);

/*
 *  Turn Stochastic Extensions messages on/off.
 *  Message msgid (use symbolic names from message table) is turned off
 *  if onoff is 0, turned on if onoff is 1.
 *
 * INPUT:
 *   long msgid                Message identifier (from message table).
 *   long onoff                0 turns off printing, 1 turns on printing
 *
 * OUTPUT:
 *   long *rc                  pointer toreturn code 2, is fatal
 *
 * RETURNS:
 *      void
 **************************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_DistributionAtTreeNode
                (long *rc, Tree *tree, long *arr,long narr);

/*
 * Finds the node corresponding to arr[0..nstg-1]
 * and reports the conditional distribution to standard output
 *
 * INPUT:
 *  Tree *tree       pointer to tree initialize by ekks_InitializeTree.
 *  long *arr[]      integer array of data values
 *  long narr        number of data values (stages) in arr[]

 *
 * OUTPUT:
 *  long *rc          return codes:
 *                      1 if maximum number of samples exceeded
 *                      2 if tree not initialized or corrupted
 *
 * RETURNS:
 *      void
 ************************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_RegisterCallBack
                (int * cbid, void * ekkcbptr);
/*
 * RETURNS:
 *      void
 ************************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_CloseCallBack(int *cbid);
/*
 * RETURNS:
 *      void
 ************************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_PassModelToOSL
                (long *rc, Stoch_Model *smodel );

/*
 *   Passes matrix, costs and bounds data to the OSL model corresponding
 *   to smodel.
 *
 *
 * INPUT:
 *   Stoch_Model *smodel         stochastic model declared by ekks_DescribeModel.
 *
 * OUTPUT:
 *   long *rc                    point return code 2 is fatal.
 *
 * RETURNS:
 *      void
 *
 ***********************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_GetDualSolution
                (long *rc, Stoch_Model *smodel, long s, long colrowtype,
                 double *solution, long *indx);

/*
 *      Put scenario duals at location pointed to by dualsout[].
 *      (Must be malloced by user.)
 *
  * INPUT:
 *  Stoch_Model *smodel          Stoch_Model or Stoch_Data pointer to be deleted
 *..long s                       Scenario number to return
 *  long colrowtype              type of solution (1=rows, else columns) to be retrieved
 *
 * OUTPUT:
 *  double *solution             pointer to the solution
 *  long *indx                   pointer to the indices
 *  long *rc                     pointer to return code, 2 is fatal

 *
 * RETURNS:
 *      void
 *****************************************************************************/

extern OSLLIBAPI void OSLLINKAGE  ekkse_NestedLSolve
                (long *rc, Stoch_Model *s_model, long minNumRows,
                 long ipcut, long maxmod, long maxcycles, double rmaxmin,
                 long inwmt, long icrsh, long iscal,
                 long isslvalg, long isslvinit, long largebounds,
                 long *status, double *optgap );

/*
 *  Applies Nested decomposition to the stochastic model s_model.
 *
 * Methodology:
 *
 *  Subproblems are created according to the assignment
 *  of nodes to subproblems.  The logic is as follows:
 *              if minNumRows > 0, nodes are accumulated into subproblems until
 *                      minNumRows is exceeded
 *              else if ipcut > 0, subproblems are generated by cutting at period ipcut
 *              else if maxmod > 0, use minNumRows = .9 * totalrows/maxmod
 *                      and distribute any excess evenly among subproblems.
 *
 *
 *  The program performs MAXCYCLES iterations of Nested decomposition,
 *  terminating if optimality, infeasibility, or unboundedness is
 *  detected.

 *
 * INPUT:
 *
 *  Stoch_Model *s_model stochastic model created by OSL.
 *  long minNumRows      min number of rows for size of subproblems
 *  long ipcut           period at which Nested cuts are made
 *  long maxmod          maximum number of subproblems
 *  long maxcycles       maximum number of Master iterations
 *  double rmaxmin       1.0=maximize, -1.0=minimize
 *  long inwmt           EKKNWMT type (0 => none EKKNWMT) for all problems
 *  long iscal           1= do, 0=don't perform scaling for each submodel
 *  long icrsh           crash type (0 => none) for first solution of submodels
 *  isslvalg             EKKSSLV algorithm for first solution of submodels
 *  isslvinit            the type of pricing procedure to be used
 *  large_bounds         1= add, 0=don't add large bounds to master
 *  *optgap              pointer to Target optimality gap
 *
 * OUTPUT:
 *
 *  long rc              return code, >= 2 indicates fatal error.
 *  status               0=optimal,1=infeasible,2=unbounded,>2=not solved
 *  optgap               pointer to relative accuracy of attained optimality gap
 *
 ************************************************************************************/
extern OSLLIBAPI void * OSLLINKAGE  ekkse_AllocateMemory(char * callingProc,
  long size, long type);

#endif /* USRTREE_H */

#endif /* USRSTOC_H
**********************************************************************/

#ifdef __cplusplus
}
#endif
