/*
 * exBender.cpp:  Main driver for reading SMPS-files and solving.
 *
 *         Reads problem from SMPS files and solves using
 *         ekks_bendersLSolve().
 *         (compile with  /DOSLMSDLL on INTEL platforms.)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "scn_c_api.h"

#define COLUMNS 0
#define ROWS    1
#define MXSCN 15000            /* maximum scenarios */

void main( long argc, char *argv[] ) {
    ekks_initialize();
    {
      EKKStoch *stoch=ekks_createStoch(NULL,15000);    /* Create an EKKStoch object */
      long type=ekks_readSMPSData(stoch,"KandW3.core","KandW3.time","KandW3.stoch");  /* Read model data from SMPS files */
      if (type<0) exit(999);
      ekks_writeNativeData(stoch,"KandW3.spl");             /* Write stochastic data in binary format */
      ekks_describeFullModel(stoch,1);                      /* Describe Full model and populate with data */
      ekks_bendersLSolve(stoch,0);                          /* Solve using bendersLSolve() -- (parm2=0) No Hotstart */
      ekks_printNodeSolution(stoch,1,1,COLUMNS);            /* Print node solution at (parm2) scenario=1, (parm3) stage=1, (parm4) COLUMNS=0 */
      ekks_deleteStoch(stoch);                              /* Delete the object */
    }
    ekks_end();
}
