/*
 * exNested.cpp:  Main driver for reading SPL-files and solving.
 *
 *         Reads problem from SPL file and solves using
 *         ekkse_nestedLSolve().
 *         (compile with  /DOSLMSDLL on INTEL platforms.)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "scn_c_api.h"

#define MXSCN 15000            /* maximum scenarios */

void main( long argc, char *argv[] ) {
    EKKContext *env=ekks_initializeContext();
    {
      EKKStoch *stoch=ekks_newStoch(env,"MyStoch",MXSCN);    /* Create an EKKStoch object */
      ekks_readNativeData(stoch,"prod_mix.spl");            /* Populate object with data from SPL file */
      ekks_setMaxiter(stoch,10);                            /* Set maximum iterations to 10 */
      ekks_describeFullModel(stoch,1);                      /* Describe the full tree model and populate with data */
      ekkse_nestedLSolve(stoch,1,0);                        /* Solve using NestedLSolve() -- (parm2=1) Cut at Stage, (parm3=0) No Hotstart */
      ekks_printObjectiveDistribution(stoch);               /* Print the distribution of scenario optimal objective values */
      ekks_deleteStoch(stoch);                              /* Delete the object */
    }
    ekks_endContext(env);
}
