/*
 * exParam.cpp:  Main driver for reading SPL-files and solving the parametric model.
 *
 *         Reads problem from SPL file and solves using
 *         ekkse_nestedLSolve(), and then solves a parametric model using
 *         ekkse_solveParametricModel.
 *         (compile with  /DOSLMSDLL on INTEL platforms.)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "scn_c_api.h"

#define MXSCN 25000            /* maximum scenarios */

long myReport(EKKModel *model) {            /* Report writing function  */
    double robj=ekk_getRobjvalue(model);
    printf("\n Obj = %g\n", robj);
    return ekk_getIprobstat(model);
}

void main( long argc, char *argv[] ) {
    int rc=0;
    EKKContext *env=ekks_initializeContext();
    {
      EKKStoch *stoch=ekks_newStoch(env,"MyStoch",MXSCN);    /* Create an EKKStoch object */
      ekks_readNativeData(stoch,"app0110.spl");              /* Populate object with data from SPL file */
      ekks_describeFullModel(stoch,1);                       /* Describe the full tree model and populate with data */
      rc=ekkse_nestedLSolve(stoch,1,0);                      /* Solve using NestedLSolve() -- (parm2=1) Cut at Stage, (parm3=0) No Hotstart */
      EKKModel * model=ekkse_getCurrentModel(stoch);         /* Obtain a handle to the full base model */

      int ncol=ekk_getInumcols(model);                       /* Obtain the number of columns for the full model */
      double *parobj=ekk_double(ncol);                       /* Allocate space for the parametric objective vector */
      for (int ii=0; ii<ncol; ii++) {                        /* Set up the parametric objective vector */
         parobj[ii]=0.01*ii;
      }
      ekkse_solveParametricModel(stoch,parobj,NULL,NULL,NULL,NULL,1,myReport);  /* Solve the parametric model */
      ekk_free(parobj);                                      /* Free the parametric objective vector */
      ekks_deleteStoch(stoch);                               /* Delete the object */
    }
    ekks_endContext(env);
}
