/*
 * exSample.cpp:  Main driver for reading SMPS-files and solving.
 *
 *         Reads problem from SMPS files and solves using
 *         ekks_bendersLSolve().
 *         (compile with  /DOSLMSDLL on INTEL platforms.)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "scn_c_api.h"

#define MXSCN 15000            /* maximum scenarios */

void main( long argc, char *argv[] ) {
    ekks_initialize();
    { long rc=0;
      EKKStoch *stoch=ekks_createStoch("MyStoch",MXSCN);    /* Create an EKKStoch object */
      ekks_readNativeData(stoch,"KandW3.spl");               /* Populate object with data from SPL file */
      rc=ekks_generateSamples(stoch,4,0);                   /* Generate 4 (parm2) samples from Uniform Random (parm3) distribution */
      assert(rc==0);
      ekks_describeFullModel(stoch,1);                      /* Describe Full Model comprising of these samples */
      ekks_bendersLSolve(stoch,0);                          /* Solve using BendersLSolve() -- (parm2=0) No Hotstart */
      ekks_printObjectiveDistribution(stoch);               /* Print Scenario Distribution */
      ekks_deleteStoch(stoch);                              /* Delete the object */
    }
    ekks_end();
}
