/*
 * exStmip.cpp:  Main driver for reading SMPS-files and solving.
 *
 *         Reads problem from SMPS files and solves using
 *         ekks_bendersLSolve/ekkse_nestedLSolve and ekks_branchAndBound
 *         (compile with  /DOSLMSDLL on INTEL platforms.)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "ekkc.h"
#include "scn_c_api.h"


#define COLUMNS 0              /* print column solutions */
#define ROWS    1              /* print row solutions */
#define MXSCN 15000            /* maximum scenarios */


long prty[]={1,2};
double uppc[]={0.001,0.001};
double dnpc[]={0.999,0.999};

void main( long argc, char *argv[] ) {
    ekks_initialize();
    {
      EKKStoch *stoch=ekks_createStoch("MyStoch",MXSCN);    /* Create an EKKStoch object */
      long type=ekks_readSMPSData(stoch,"sizes.core","sizes.time","sizes.stoch");  /* Read model data from SMPS files */
      if (type<0) exit(999);

      ekks_describeFullModel(stoch,1);                      /* Describe Full model and populate with data */
      ekks_bendersLSolve(stoch,0);                          /* Solve using bendersLSolve() -- (parm2=0) No Hotstart */

      long numints=ekks_markIntegersWithStagePriorities(stoch, prty, uppc, dnpc);                /* Mark the integers in the full stochastic LP model */
      printf("\n There are %d integers \n", numints);

      /* Get base model */
      EKKModel *model=ekkse_getCurrentModel(stoch);         /* Get base model */

      ekk_messagePrintOff(model,87);                        /* Turn off printing of some messages */
      ekk_messagePrintOff(model,100);
      ekk_messagePrintOff(model,102);

      ekk_setImaxsols(model,5);                             /* Stop after at most 5 integer solutions */
      ekks_branchAndBound(stoch,NULL,NULL);                 /* Solve by branch-and-bound */

      ekks_printNodeSolution(stoch,1,1,COLUMNS);            /* Print node solution at (parm2) scenario=1, (parm3) stage=1, (parm4) COLUMNS=0 */
      ekks_deleteStoch(stoch);                              /* Delete the object */
    }
    ekks_end();
}
